# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: android_env/proto/raw_observation.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'android_env/proto/raw_observation.proto\x12\x0b\x61ndroid_env\"\xa5\x01\n\x0eRawObservation\x12\x14\n\x0ctimestamp_us\x18\x01 \x01(\x03\x12\x31\n\x06screen\x18\x02 \x01(\x0b\x32!.android_env.RawObservation.Image\x1aJ\n\x05Image\x12\r\n\x05width\x18\x01 \x01(\x05\x12\x0e\n\x06height\x18\x02 \x01(\x05\x12\x14\n\x0cnum_channels\x18\x03 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x04 \x01(\x0c\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'android_env.proto.raw_observation_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _RAWOBSERVATION._serialized_start=57
  _RAWOBSERVATION._serialized_end=222
  _RAWOBSERVATION_IMAGE._serialized_start=148
  _RAWOBSERVATION_IMAGE._serialized_end=222
# @@protoc_insertion_point(module_scope)
