# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: android_env/proto/adb.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x61ndroid_env/proto/adb.proto\x12\x0b\x61ndroid_env\"\x85\x10\n\x07\x41\x64\x62\x43\x61ll\x12\x36\n\x0binstall_apk\x18\x01 \x01(\x0b\x32\x1f.android_env.AdbCall.InstallApkH\x00\x12<\n\x0estart_activity\x18\x02 \x01(\x0b\x32\".android_env.AdbCall.StartActivityH\x00\x12\x34\n\nforce_stop\x18\x03 \x01(\x0b\x32\x1e.android_env.AdbCall.ForceStopH\x00\x12\x36\n\x0b\x63lear_cache\x18\x04 \x01(\x0b\x32\x1f.android_env.AdbCall.ClearCacheH\x00\x12\x42\n\x11grant_permissions\x18\x05 \x01(\x0b\x32%.android_env.AdbCall.GrantPermissionsH\x00\x12\'\n\x03tap\x18\x06 \x01(\x0b\x32\x18.android_env.AdbCall.TapH\x00\x12\x38\n\x0cpress_button\x18\x07 \x01(\x0b\x32 .android_env.AdbCall.PressButtonH\x00\x12-\n\x06rotate\x18\x08 \x01(\x0b\x32\x1b.android_env.AdbCall.RotateH\x00\x12U\n\x1bstart_accessibility_service\x18\t \x01(\x0b\x32..android_env.AdbCall.StartAccessibilityServiceH\x00\x12G\n\x14start_screen_pinning\x18\n \x01(\x0b\x32\'.android_env.AdbCall.StartScreenPinningH\x00\x12I\n\x15start_random_activity\x18\x0b \x01(\x0b\x32(.android_env.AdbCall.StartRandomActivityH\x00\x12R\n\x1a\x66orce_stop_random_activity\x18\x0c \x01(\x0b\x32,.android_env.AdbCall.ForceStopRandomActivityH\x00\x12T\n\x1b\x63lear_cache_random_activity\x18\r \x01(\x0b\x32-.android_env.AdbCall.ClearCacheRandomActivityH\x00\x12\x44\n\x12\x64isable_animations\x18\x0e \x01(\x0b\x32&.android_env.AdbCall.DisableAnimationsH\x00\x12\x38\n\x0cstart_intent\x18\x0f \x01(\x0b\x32 .android_env.AdbCall.StartIntentH\x00\x12\x42\n\x11uninstall_package\x18\x10 \x01(\x0b\x32%.android_env.AdbCall.UninstallPackageH\x00\x1av\n\nInstallApk\x12@\n\nfilesystem\x18\x02 \x01(\x0b\x32*.android_env.AdbCall.InstallApk.FilesystemH\x00\x1a\x1a\n\nFilesystem\x12\x0c\n\x04path\x18\x01 \x01(\tB\n\n\x08location\x1aH\n\rStartActivity\x12\x15\n\rfull_activity\x18\x01 \x01(\t\x12\x12\n\nextra_args\x18\x02 \x03(\t\x12\x0c\n\x04\x66lag\x18\x03 \x01(\t\x1a\x45\n\x0bStartIntent\x12\x10\n\x08\x64\x61ta_uri\x18\x01 \x01(\t\x12\x0e\n\x06\x61\x63tion\x18\x02 \x01(\t\x12\x14\n\x0cpackage_name\x18\x03 \x01(\t\x1aU\n\x13StartRandomActivity\x12\x15\n\ractivity_list\x18\x01 \x03(\t\x12\x12\n\nextra_args\x18\x02 \x03(\t\x12\x13\n\x0btimeout_sec\x18\x04 \x01(\x02\x1a(\n\x10UninstallPackage\x12\x14\n\x0cpackage_name\x18\x01 \x01(\t\x1a!\n\tForceStop\x12\x14\n\x0cpackage_name\x18\x01 \x01(\t\x1a\x19\n\x17\x46orceStopRandomActivity\x1a\x13\n\x11\x44isableAnimations\x1a\"\n\nClearCache\x12\x14\n\x0cpackage_name\x18\x01 \x01(\t\x1a\x1a\n\x18\x43learCacheRandomActivity\x1a=\n\x10GrantPermissions\x12\x14\n\x0cpackage_name\x18\x01 \x01(\t\x12\x13\n\x0bpermissions\x18\x02 \x03(\t\x1a\x1b\n\x03Tap\x12\t\n\x01x\x18\x01 \x01(\x05\x12\t\n\x01y\x18\x02 \x01(\x05\x1a\x64\n\x0bPressButton\x12\x37\n\x06\x62utton\x18\x01 \x01(\x0e\x32\'.android_env.AdbCall.PressButton.Button\"\x1c\n\x06\x42utton\x12\x08\n\x04HOME\x10\x00\x12\x08\n\x04\x42\x41\x43K\x10\x01\x1a\x9c\x01\n\x06Rotate\x12<\n\x0borientation\x18\x01 \x01(\x0e\x32\'.android_env.AdbCall.Rotate.Orientation\"T\n\x0bOrientation\x12\x0e\n\nPORTRAIT_0\x10\x00\x12\x10\n\x0cLANDSCAPE_90\x10\x01\x12\x10\n\x0cPORTRAIT_180\x10\x02\x12\x11\n\rLANDSCAPE_270\x10\x03\x1a\x31\n\x19StartAccessibilityService\x12\x14\n\x0c\x66ull_service\x18\x01 \x01(\t\x1a+\n\x12StartScreenPinning\x12\x15\n\rfull_activity\x18\x01 \x01(\tB\t\n\x07\x63ommandb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'android_env.proto.adb_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _ADBCALL._serialized_start=45
  _ADBCALL._serialized_end=2098
  _ADBCALL_INSTALLAPK._serialized_start=1099
  _ADBCALL_INSTALLAPK._serialized_end=1217
  _ADBCALL_INSTALLAPK_FILESYSTEM._serialized_start=1179
  _ADBCALL_INSTALLAPK_FILESYSTEM._serialized_end=1205
  _ADBCALL_STARTACTIVITY._serialized_start=1219
  _ADBCALL_STARTACTIVITY._serialized_end=1291
  _ADBCALL_STARTINTENT._serialized_start=1293
  _ADBCALL_STARTINTENT._serialized_end=1362
  _ADBCALL_STARTRANDOMACTIVITY._serialized_start=1364
  _ADBCALL_STARTRANDOMACTIVITY._serialized_end=1449
  _ADBCALL_UNINSTALLPACKAGE._serialized_start=1451
  _ADBCALL_UNINSTALLPACKAGE._serialized_end=1491
  _ADBCALL_FORCESTOP._serialized_start=1493
  _ADBCALL_FORCESTOP._serialized_end=1526
  _ADBCALL_FORCESTOPRANDOMACTIVITY._serialized_start=1528
  _ADBCALL_FORCESTOPRANDOMACTIVITY._serialized_end=1553
  _ADBCALL_DISABLEANIMATIONS._serialized_start=1555
  _ADBCALL_DISABLEANIMATIONS._serialized_end=1574
  _ADBCALL_CLEARCACHE._serialized_start=1576
  _ADBCALL_CLEARCACHE._serialized_end=1610
  _ADBCALL_CLEARCACHERANDOMACTIVITY._serialized_start=1612
  _ADBCALL_CLEARCACHERANDOMACTIVITY._serialized_end=1638
  _ADBCALL_GRANTPERMISSIONS._serialized_start=1640
  _ADBCALL_GRANTPERMISSIONS._serialized_end=1701
  _ADBCALL_TAP._serialized_start=1703
  _ADBCALL_TAP._serialized_end=1730
  _ADBCALL_PRESSBUTTON._serialized_start=1732
  _ADBCALL_PRESSBUTTON._serialized_end=1832
  _ADBCALL_PRESSBUTTON_BUTTON._serialized_start=1804
  _ADBCALL_PRESSBUTTON_BUTTON._serialized_end=1832
  _ADBCALL_ROTATE._serialized_start=1835
  _ADBCALL_ROTATE._serialized_end=1991
  _ADBCALL_ROTATE_ORIENTATION._serialized_start=1907
  _ADBCALL_ROTATE_ORIENTATION._serialized_end=1991
  _ADBCALL_STARTACCESSIBILITYSERVICE._serialized_start=1993
  _ADBCALL_STARTACCESSIBILITYSERVICE._serialized_end=2042
  _ADBCALL_STARTSCREENPINNING._serialized_start=2044
  _ADBCALL_STARTSCREENPINNING._serialized_end=2087
# @@protoc_insertion_point(module_scope)
