# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: android_env/proto/task.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from android_env.proto import adb_pb2 as android__env_dot_proto_dot_adb__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1c\x61ndroid_env/proto/task.proto\x12\x0b\x61ndroid_env\x1a\x1b\x61ndroid_env/proto/adb.proto\":\n\tAppScreen\x12\x10\n\x08\x61\x63tivity\x18\x01 \x01(\t\x12\x1b\n\x13view_hierarchy_path\x18\x02 \x03(\t\"S\n\x10WaitForAppScreen\x12*\n\napp_screen\x18\x01 \x01(\x0b\x32\x16.android_env.AppScreen\x12\x13\n\x0btimeout_sec\x18\x02 \x01(\x02\"9\n\x0c\x43heckInstall\x12\x14\n\x0cpackage_name\x18\x01 \x01(\t\x12\x13\n\x0btimeout_sec\x18\x02 \x01(\x02\"6\n\x0eWaitForMessage\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x13\n\x0btimeout_sec\x18\x02 \x01(\x02\"\x19\n\x05Sleep\x12\x10\n\x08time_sec\x18\x01 \x01(\x02\"\xdb\x01\n\x10SuccessCondition\x12\x13\n\x0bnum_retries\x18\x01 \x01(\x05\x12<\n\x13wait_for_app_screen\x18\x02 \x01(\x0b\x32\x1d.android_env.WaitForAppScreenH\x00\x12\x32\n\rcheck_install\x18\x03 \x01(\x0b\x32\x19.android_env.CheckInstallH\x00\x12\x37\n\x10wait_for_message\x18\x04 \x01(\x0b\x32\x1b.android_env.WaitForMessageH\x00\x42\x07\n\x05\x63heck\"\x9c\x01\n\tSetupStep\x12\x38\n\x11success_condition\x18\x01 \x01(\x0b\x32\x1d.android_env.SuccessCondition\x12(\n\x08\x61\x64\x62_call\x18\x02 \x01(\x0b\x32\x14.android_env.AdbCallH\x00\x12#\n\x05sleep\x18\x03 \x01(\x0b\x32\x12.android_env.SleepH\x00\x42\x06\n\x04step\"\xb7\x02\n\tArraySpec\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05shape\x18\x02 \x03(\x05\x12.\n\x05\x64type\x18\x03 \x01(\x0e\x32\x1f.android_env.ArraySpec.DataType\"\xdc\x01\n\x08\x44\x61taType\x12\x15\n\x11INVALID_DATA_TYPE\x10\x00\x12\t\n\x05\x46LOAT\x10\x01\x12\n\n\x06\x44OUBLE\x10\x02\x12\x08\n\x04INT8\x10\x03\x12\t\n\x05INT16\x10\x04\x12\t\n\x05INT32\x10\x05\x12\t\n\x05INT64\x10\x06\x12\t\n\x05UINT8\x10\x07\x12\n\n\x06UINT16\x10\x08\x12\n\n\x06UINT32\x10\t\x12\n\n\x06UINT64\x10\n\x12\x08\n\x04\x42OOL\x10\x0b\x12\r\n\tSTRING_U1\x10\x0c\x12\x0e\n\nSTRING_U16\x10\r\x12\x0e\n\nSTRING_U25\x10\x0e\x12\x0f\n\x0bSTRING_U250\x10\x0f\"\xcc\n\n\x0b\x45ventSource\x12<\n\x0etext_recognize\x18\x03 \x01(\x0b\x32\".android_env.EventSource.TextEventH\x00\x12\x39\n\x0btext_detect\x18\x04 \x01(\x0b\x32\".android_env.EventSource.TextEventH\x00\x12\x41\n\x0eicon_recognize\x18\x05 \x01(\x0b\x32\'.android_env.EventSource.IconRecogEventH\x00\x12>\n\x0bicon_detect\x18\x06 \x01(\x0b\x32\'.android_env.EventSource.IconRecogEventH\x00\x12=\n\nicon_match\x18\x07 \x01(\x0b\x32\'.android_env.EventSource.IconMatchEventH\x00\x12\x44\n\x11icon_detect_match\x18\x08 \x01(\x0b\x32\'.android_env.EventSource.IconMatchEventH\x00\x12K\n\x14view_hierarchy_event\x18\t \x01(\x0b\x32+.android_env.EventSource.ViewHierarchyEventH\x00\x12\x36\n\tlog_event\x18\n \x01(\x0b\x32!.android_env.EventSource.LogEventH\x00\x12\n\n\x02id\x18\x0c \x01(\x05\x12=\n\rrepeatability\x18\x0e \x01(\x0e\x32&.android_env.EventSource.Repeatability\x1a\x36\n\x04Rect\x12\n\n\x02x0\x18\x02 \x01(\x02\x12\n\n\x02y0\x18\x03 \x01(\x02\x12\n\n\x02x1\x18\x04 \x01(\x02\x12\n\n\x02y1\x18\x05 \x01(\x02\x1aH\n\tTextEvent\x12\x0e\n\x06\x65xpect\x18\x01 \x01(\t\x12+\n\x04rect\x18\x02 \x01(\x0b\x32\x1d.android_env.EventSource.Rect\x1aL\n\x0eIconRecogEvent\x12\r\n\x05\x63lass\x18\x01 \x01(\t\x12+\n\x04rect\x18\x02 \x01(\x0b\x32\x1d.android_env.EventSource.Rect\x1aK\n\x0eIconMatchEvent\x12\x0c\n\x04path\x18\x01 \x01(\t\x12+\n\x04rect\x18\x02 \x01(\x0b\x32\x1d.android_env.EventSource.Rect\x1a\xe3\x02\n\x12ViewHierarchyEvent\x12\x1b\n\x13view_hierarchy_path\x18\x01 \x03(\t\x12H\n\nproperties\x18\x02 \x03(\x0b\x32\x34.android_env.EventSource.ViewHierarchyEvent.Property\x1a\xad\x01\n\x08Property\x12\x15\n\rproperty_name\x18\x01 \x01(\t\x12>\n\x04sign\x18\x02 \x01(\x0e\x32\x30.android_env.EventSource.ViewHierarchyEvent.Sign\x12\x11\n\x07pattern\x18\x03 \x01(\tH\x00\x12\x11\n\x07integer\x18\x04 \x01(\x05H\x00\x12\x12\n\x08\x66loating\x18\x05 \x01(\x02H\x00\x42\x10\n\x0eproperty_value\"6\n\x04Sign\x12\x06\n\x02\x45Q\x10\x00\x12\x06\n\x02LE\x10\x01\x12\x06\n\x02LT\x10\x02\x12\x06\n\x02GE\x10\x03\x12\x06\n\x02GT\x10\x04\x12\x06\n\x02NE\x10\x05\x1a,\n\x08LogEvent\x12\x0f\n\x07\x66ilters\x18\x01 \x03(\t\x12\x0f\n\x07pattern\x18\x02 \x01(\t\"2\n\rRepeatability\x12\x08\n\x04NONE\x10\x00\x12\x08\n\x04LAST\x10\x01\x12\r\n\tUNLIMITED\x10\x02\x42\x07\n\x05\x65vent\"\x97\x02\n\tEventSlot\x12)\n\x04type\x18\x01 \x01(\x0e\x32\x1b.android_env.EventSlot.Type\x12\n\n\x02id\x18\x02 \x01(\x05\x12\x31\n\x06\x65vents\x18\x03 \x03(\x0b\x32!.android_env.EventSlot.FakeSource\x12\x14\n\x0cprerequisite\x18\x04 \x03(\x05\x12\x16\n\x0etransformation\x18\x05 \x03(\t\x1aM\n\nFakeSource\x12\x0c\n\x02id\x18\x01 \x01(\x05H\x00\x12\'\n\x05\x65vent\x18\x02 \x01(\x0b\x32\x16.android_env.EventSlotH\x00\x42\x08\n\x06source\"#\n\x04Type\x12\n\n\x06SINGLE\x10\x00\x12\x07\n\x03\x41ND\x10\x01\x12\x06\n\x02OR\x10\x02\"\xc2\x02\n\x0eTaskEventSlots\x12.\n\x0escore_listener\x18\x01 \x01(\x0b\x32\x16.android_env.EventSlot\x12/\n\x0freward_listener\x18\x02 \x01(\x0b\x32\x16.android_env.EventSlot\x12\x34\n\x14\x65pisode_end_listener\x18\x03 \x01(\x0b\x32\x16.android_env.EventSlot\x12.\n\x0e\x65xtra_listener\x18\x04 \x01(\x0b\x32\x16.android_env.EventSlot\x12\x33\n\x13json_extra_listener\x18\x05 \x01(\x0b\x32\x16.android_env.EventSlot\x12\x34\n\x14instruction_listener\x18\x06 \x01(\x0b\x32\x16.android_env.EventSlot\"\xaa\x03\n\x04Task\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12+\n\x0bsetup_steps\x18\x04 \x03(\x0b\x32\x16.android_env.SetupStep\x12+\n\x0breset_steps\x18\x05 \x03(\x0b\x32\x16.android_env.SetupStep\x12\x33\n\x13\x65xpected_app_screen\x18\x06 \x01(\x0b\x32\x16.android_env.AppScreen\x12\x18\n\x10max_duration_sec\x18\x07 \x01(\x02\x12\x15\n\rmax_num_steps\x18\x08 \x01(\x05\x12/\n\revent_sources\x18\t \x03(\x0b\x32\x18.android_env.EventSource\x12\x30\n\x0b\x65vent_slots\x18\n \x01(\x0b\x32\x1b.android_env.TaskEventSlots\x12+\n\x0b\x65xtras_spec\x18\x0b \x03(\x0b\x32\x16.android_env.ArraySpec\x12\x0f\n\x07\x63ommand\x18\x0c \x03(\t\x12\x12\n\nvocabulary\x18\r \x03(\tb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'android_env.proto.task_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _APPSCREEN._serialized_start=74
  _APPSCREEN._serialized_end=132
  _WAITFORAPPSCREEN._serialized_start=134
  _WAITFORAPPSCREEN._serialized_end=217
  _CHECKINSTALL._serialized_start=219
  _CHECKINSTALL._serialized_end=276
  _WAITFORMESSAGE._serialized_start=278
  _WAITFORMESSAGE._serialized_end=332
  _SLEEP._serialized_start=334
  _SLEEP._serialized_end=359
  _SUCCESSCONDITION._serialized_start=362
  _SUCCESSCONDITION._serialized_end=581
  _SETUPSTEP._serialized_start=584
  _SETUPSTEP._serialized_end=740
  _ARRAYSPEC._serialized_start=743
  _ARRAYSPEC._serialized_end=1054
  _ARRAYSPEC_DATATYPE._serialized_start=834
  _ARRAYSPEC_DATATYPE._serialized_end=1054
  _EVENTSOURCE._serialized_start=1057
  _EVENTSOURCE._serialized_end=2413
  _EVENTSOURCE_RECT._serialized_start=1665
  _EVENTSOURCE_RECT._serialized_end=1719
  _EVENTSOURCE_TEXTEVENT._serialized_start=1721
  _EVENTSOURCE_TEXTEVENT._serialized_end=1793
  _EVENTSOURCE_ICONRECOGEVENT._serialized_start=1795
  _EVENTSOURCE_ICONRECOGEVENT._serialized_end=1871
  _EVENTSOURCE_ICONMATCHEVENT._serialized_start=1873
  _EVENTSOURCE_ICONMATCHEVENT._serialized_end=1948
  _EVENTSOURCE_VIEWHIERARCHYEVENT._serialized_start=1951
  _EVENTSOURCE_VIEWHIERARCHYEVENT._serialized_end=2306
  _EVENTSOURCE_VIEWHIERARCHYEVENT_PROPERTY._serialized_start=2077
  _EVENTSOURCE_VIEWHIERARCHYEVENT_PROPERTY._serialized_end=2250
  _EVENTSOURCE_VIEWHIERARCHYEVENT_SIGN._serialized_start=2252
  _EVENTSOURCE_VIEWHIERARCHYEVENT_SIGN._serialized_end=2306
  _EVENTSOURCE_LOGEVENT._serialized_start=2308
  _EVENTSOURCE_LOGEVENT._serialized_end=2352
  _EVENTSOURCE_REPEATABILITY._serialized_start=2354
  _EVENTSOURCE_REPEATABILITY._serialized_end=2404
  _EVENTSLOT._serialized_start=2416
  _EVENTSLOT._serialized_end=2695
  _EVENTSLOT_FAKESOURCE._serialized_start=2581
  _EVENTSLOT_FAKESOURCE._serialized_end=2658
  _EVENTSLOT_TYPE._serialized_start=2660
  _EVENTSLOT_TYPE._serialized_end=2695
  _TASKEVENTSLOTS._serialized_start=2698
  _TASKEVENTSLOTS._serialized_end=3020
  _TASK._serialized_start=3023
  _TASK._serialized_end=3449
# @@protoc_insertion_point(module_scope)
